/* This file is Copyright 2001 Level Control Systems.  See the included LICENSE.txt file for details. */

#ifndef MEMORY_BUFFER_DATA_IO
#define MEMORY_BUFFER_DATA_IO

#include "dataio/PortableDataIO.h"

/**
 *  Data I/O equivalent for reading from/writing in-memory arrays
 */
class MemoryBufferDataIO : public PortableDataIO
{
public:
   /** Constructor.
    *  @param readBuf A byte buffer to read from.  May be NULL if no readable data is to be provided.  Ownership is retained by caller.
    *  @param readBufSize Number of bytes pointed to by (readBuf), or zero if (readBuf) is NULL.
    *  @param writeBuf A byte buffer to write to.  May be NULL if no write capability is to be provided.  Ownership is retained by caller.
    *  @param writeBufSize Number of bytes pointed to by (writeBuf), or zero if (writeBuf) is NULL.
    */
   MemoryBufferDataIO(const uint8 * readBuf, size_t readBufSize, uint8 * writeBuf, size_t writeBufSize) : _readBuf(readBuf), _readBufSize(readBufSize), _readBufOffset(0), _maxReadChunk(-1), _writeBuf(writeBuf), _writeBufSize(writeBufSize), _writeBufOffset(0), _maxWriteChunk(-1)  {/* empty */}

   /** Virtual Destructor, to keep C++ honest */
   virtual ~MemoryBufferDataIO() {/* empty */}

   /** 
    *  Copies bytes from our read buffer into (buffer).  If we have no read buffer, or no bytes left to read, returns -1.
    *  @param buffer Points to a buffer to read bytes into.
    *  @param size Number of bytes in the buffer.
    *  @return zero.
    */
   virtual ssize_t Read(void * buffer, size_t size)  
   {
      ssize_t readBytesAvailable = _readBuf ? (_readBufSize - _readBufOffset) : -1;
      ssize_t copyBytes = (size < readBytesAvailable) ? size : readBytesAvailable;
      if (copyBytes > 0) 
      {
         if ((_maxReadChunk >= 0)&&(copyBytes > _maxReadChunk)) copyBytes = _maxReadChunk;
         memcpy(buffer, &_readBuf[_readBufOffset], copyBytes);
         _readBufOffset += copyBytes;
         return copyBytes;
      }
      return -1;
   }

   /** 
    *  Writes bytes into our write buffer.  If we have no write buffer, or there is no room left in the write buffer, returns -1.
    *  @param buffer Points to a buffer to write bytes from.
    *  @param size Number of bytes in the buffer.
    *  @return (size).
    */
   virtual ssize_t Write(const void * buffer, size_t size) 
   {
      ssize_t writeBytesAvailable = _writeBuf ? (_writeBufSize - _writeBufOffset) : -1;
      ssize_t copyBytes = (size < writeBytesAvailable) ? size : writeBytesAvailable;
      if (copyBytes > 0) 
      {
         if ((_maxWriteChunk >= 0)&&(copyBytes > _maxWriteChunk)) copyBytes = _maxWriteChunk;
         memcpy(&_writeBuf[_writeBufOffset], buffer, copyBytes);
         _writeBufOffset += copyBytes;
         return copyBytes;
      }
      return -1;
   }

   /** 
    *  No-op method.
    *  This method doesn't do anything at all.
    */
   virtual void FlushOutput() {/* empty */}

   /** Disable us! */ 
   virtual void Shutdown() {_readBuf = _writeBuf = NULL;}

   /** For testing--you can call this to artificially limit the maximum number of bytes returnable in a single call to Read().
    *  @param mrc Maximum read chunk size, in bytes.  If less than zero, then no fixed limit is enforced (default behaviour)
    */
   void SetMaxReadChunk(ssize_t mrc) {_maxReadChunk = mrc;}

   /** For testing--you can call this to artificially limit the maximum number of bytes copyable in a single call to Write().
    *  @param mwc Maximum write chunk size, in bytes.  If less than zero, then no fixed limit is enforced (default behaviour)
    */
   void SetMaxWriteChunk(ssize_t mwc) {_maxWriteChunk = mwc;}

private:
   const uint8 * _readBuf;
   ssize_t _readBufSize;
   ssize_t _readBufOffset;
   ssize_t _maxReadChunk;

   uint8 * _writeBuf;
   ssize_t _writeBufSize;
   ssize_t _writeBufOffset;
   ssize_t _maxWriteChunk;
};

#endif
